from cpl_core.application import ApplicationExtensionABC
from cpl_core.configuration import ConfigurationABC
from cpl_core.dependency_injection import ServiceProviderABC
from cpl_translation import TranslatePipe

from bot_core.abc.client_utils_service_abc import ClientUtilsServiceABC
from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum
from bot_core.configuration.feature_flags_settings import FeatureFlagsSettings
from bot_core.helper.command_checks import CommandChecks
from bot_core.helper.event_checks import EventChecks
from modules.permission.abc.permission_service_abc import PermissionServiceABC


class CoreExtension(ApplicationExtensionABC):

    def __init__(self):
        ApplicationExtensionABC.__init__(self)

    async def run(self, config: ConfigurationABC, services: ServiceProviderABC):
        feature_flags: FeatureFlagsSettings = config.get_configuration(FeatureFlagsSettings)
        if not feature_flags.get_flag(FeatureFlagsEnum.core_module):
            return

        permissions: PermissionServiceABC = services.get_service(PermissionServiceABC)
        client_utils: ClientUtilsServiceABC = services.get_service(ClientUtilsServiceABC)
        message_service: MessageServiceABC = services.get_service(MessageServiceABC)
        t: TranslatePipe = services.get_service(TranslatePipe)
        CommandChecks.init(permissions, client_utils, message_service, t)
        EventChecks.init(client_utils)
