from cpl_core.database.context import DatabaseContextABC
from cpl_core.logging import LoggerABC
from cpl_discord.service import DiscordBotServiceABC
from discord import RawReactionActionEvent

from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from modules.base.abc.base_helper_abc import BaseHelperABC
from modules.base.configuration.base_server_settings import BaseServerSettings


class BaseReactionHandler:

    def __init__(
            self,
            logger: LoggerABC,
            bot: DiscordBotServiceABC,
            servers: ServerRepositoryABC,
            users: UserRepositoryABC,
            base_helper: BaseHelperABC,
            db: DatabaseContextABC,
    ):
        self._logger = logger
        self._bot = bot
        self._servers = servers
        self._users = users
        self._base_helper = base_helper
        self._db = db

    async def handle(self, payload: RawReactionActionEvent, r_type=None) -> None:
        self._logger.trace(__name__, f'Handle reaction {payload} {r_type}')

        guild = self._bot.get_guild(payload.guild_id)
        member = guild.get_member(payload.user_id)
        if member is None:
            self._logger.warn(__name__, f'User {payload.user_id} in {guild.name} not found - skipping')
            return

        server = self._servers.get_server_by_discord_id(guild.id)
        user = self._users.get_user_by_discord_id_and_server_id(member.id, server.server_id)
        settings: BaseServerSettings = self._base_helper.get_config(guild.id)

        if r_type == 'add':
            user.xp += settings.xp_per_reaction
            self._users.update_user(user)
            self._db.save_changes()
        elif r_type == 'remove':
            user.xp -= settings.xp_per_reaction
            self._users.update_user(user)
            self._db.save_changes()
        else:
            self._logger.warn(__name__, f'Invalid reaction type {r_type}')
