from cpl_core.configuration import ConfigurationABC
from cpl_core.dependency_injection import ServiceProviderABC
from cpl_discord.application import DiscordBotApplicationABC
from cpl_discord.configuration import DiscordBotSettings
from cpl_discord.service import DiscordBotServiceABC
from cpl_translation import TranslationSettings, TranslationServiceABC

from bot_core.configuration.bot_settings import BotSettings


class TestApplication(DiscordBotApplicationABC):

    def __init__(self, config: ConfigurationABC, services: ServiceProviderABC):
        DiscordBotApplicationABC.__init__(self, config, services)

        self._config = config
        self._services = services

        self._bot: DiscordBotServiceABC = services.get_service(DiscordBotServiceABC)
        self._translation: TranslationServiceABC = services.get_service(TranslationServiceABC)

        discord_bot_settings: DiscordBotSettings = config.get_configuration(DiscordBotSettings)
        self._discord_settings = self._get_settings(discord_bot_settings)

    def _get_settings(self, settings_from_config: DiscordBotSettings) -> DiscordBotSettings:
        new_settings = DiscordBotSettings()
        token = None if settings_from_config is None else settings_from_config.token
        prefix = None if settings_from_config is None else settings_from_config.prefix
        env_token = self._config.get_configuration('TOKEN')
        env_prefix = self._config.get_configuration('PREFIX')

        new_settings.from_dict({
            'Token': env_token if token is None or token == '' else token,
            'Prefix': ('! ' if self._is_string_invalid(env_prefix) else env_prefix) if self._is_string_invalid(prefix) else prefix
        })
        if new_settings.token is None or new_settings.token == '':
            raise Exception('You have to configure discord token by appsettings or environment variables')
        return new_settings

    @staticmethod
    def _is_string_invalid(x):
        return x is None or x == ''

    @property
    def config(self) -> ConfigurationABC:
        return self._config

    @property
    def services(self) -> ServiceProviderABC:
        return self._services

    async def configure(self):
        self._translation.load_by_settings(self._configuration.get_configuration(TranslationSettings))

    async def main(self):
        if self._config.get_configuration('IS_UNITTEST'):
            await self._bot.login(self._discord_settings.token)
            self._bot.loop.create_task(self._bot.connect())
            return

        await self._bot.start_async()

    async def stop_async(self):
        await self._bot.close()
